# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# GastoXProyecto: relaciona gasto con proyecto y el porcentaje de esta relación

class GastoXProyecto < ActiveRecord::Base

  #untranslate_all

  # Metemos la validacion antes para que no se eliminen gxp de proyectos cerrados
  before_destroy :verifica_borrado

  # acts_as_reportable
  belongs_to :gasto
  belongs_to :proyecto
  belongs_to :subpartida
  belongs_to :marcado, :class_name => "Marcado", :foreign_key => "marcado_proyecto_id"
  belongs_to :tasa_cambio_proyecto, :class_name => "TasaCambio", :foreign_key => "tasa_cambio_id"
  # has_many :gasto_x_actividad, :through => :gasto
  # has_many :gasto_x_agente, :through => :gasto

  # Permitimos gasto_x_proyecto vacio para porcentaje de gastos asumidos por delegacion
  #validates_presence_of :proyecto_id, :message => _("Proyecto") + " " + _("no puede estar vacío.")
  #validates_associated :proyecto, :message => _("El proyecto asociado no es correcto.")
  validates_presence_of :gasto_id, :message => _("Gasto") + " " + _("no puede estar vacío.")
  #validates_associated :gasto, :message =>  _("El gasto asociado no es correcto.")
  validates_uniqueness_of :proyecto_id, :scope => :gasto_id, :message => _("Asignación del gasto repetida.")

  validate :comprueba_importe, :comprueba_implementador_proyecto, :verifica_plugins
  validate :comprobar_estado_proyecto, :comprobar_fecha_etapas_proyecto, :comprobar_periodos_proyecto

  before_save :obtiene_tasa_cambio
  # Este callback es en create para evitar cambiar cosas modificadas o no seguir validaciones de gastos
  after_create :asigna_subpartida_por_defecto

  # Para que los plugins puedan modificar saltandose la validacion "verifica_plugins"
  # OJO!: Siempre que se hagan movimientos en los gastos desde migraciones hay que tener en cuenta esto y
  #       actualizar el gasto incluyendo "evitar_validacion_plugins: true"!
  attr_accessor :evitar_validacion_plugins


  # Comprueba que el proyecto tenga como implementador al implementador del gasto
  def comprueba_implementador_proyecto
    errors.add("proyecto", _("El proyecto '%{proy}' no tiene asignado el implementador del gasto '%{gsto}'")%{proy: proyecto.nombre, gsto: gasto.agente.nombre}) unless proyecto.nil? || proyecto.implementador.find_by_id(gasto.agente_id)
    return errors.empty?
  end

  # Comprueba que la fecha del gasto está dentro de alguna etapa del proyecto
  def comprobar_fecha_etapas_proyecto
    # Solo comprobamos si hay proyecto (proyecto_id == nil => imputado al agente)
    if proyecto
      # Comprueba que el gasto entre dentro de las fechas de proyecto
      p_fecha_inicio = proyecto.fecha_de_inicio
      p_fecha_fin = proyecto.fecha_de_fin
      unless p_fecha_inicio and p_fecha_fin and gasto.fecha >= p_fecha_inicio and gasto.fecha <= p_fecha_fin
       errors.add("fecha", _("Proyecto '%s' no tiene definidas fechas de comienzo o de fin") % [proyecto.nombre]) unless p_fecha_inicio and p_fecha_fin
       errors.add("fecha", _("La fecha '%s' no esta dentro de las etapas del proyecto '%s'") % [gasto.fecha,proyecto.nombre]) if p_fecha_inicio and p_fecha_fin
      end
    end
    return errors.empty?
  end

  # Comprueba que el proyecto no este cerrado
  def comprobar_estado_proyecto
    # Solo comprobamos si hay proyecto (proyecto_id == nil => imputado al agente) y si este tiene estado
    if proyecto && proyecto.estado_actual
      errors.add(_("Proyecto"),  _("El proyecto '%{proy}' se encuentra en estado '%{est}'.")%{proy: proyecto.nombre, est: proyecto.estado_actual.definicion_estado.nombre} + " " +
                                 _("En este estado no se puede modificar el gasto.")) if proyecto.estado_actual.definicion_estado.cerrado || !proyecto.estado_actual.definicion_estado.ejecucion
    end
    return errors.empty?
  end

  def comprobar_periodos_proyecto
    if proyecto
      for periodo in proyecto.periodo
        if periodo.gastos_cerrados and (gasto.fecha >= periodo.fecha_inicio and gasto.fecha <= periodo.fecha_fin) 
          texto = _("El gasto está asociado al proyecto: %{proy}.")%{ proy: proyecto.nombre } + 
                  _("El periodo de justificación: %{per} ( + %{inicio}  / + %{fin}) tiene gastos cerrado y no se puede modificar el gasto.")%{per: periodo.tipo_periodo.nombre, inicio: periodo.fecha_inicio.to_s , fin: periodo.fecha_fin.to_s }
          errors.add(_("Proyecto"), texto) 
        end
      end
    end
    return false unless self.errors.empty?
  end


  def comprueba_importe
    errors.add("GastoXProyecto", _("El gasto por proyecto no puede ser mayor que el total.")) if importe && importe.round(2) > gasto.importe 
    errors.add("GastoXProyecto", _("El importe vinculado al proyecto no puede estar vacío.")) unless importe
  end

  def obtiene_tasa_cambio
    # Solo actua si es nuevo o se ha cambiado el proyecto o el gasto (los cambios en moneda o fecha los hace el propio gasto)
    if self.id.nil? || self.gasto_id_changed? || self.proyecto_id_changed?
      tc = TasaCambio.tasa_cambio_para_gasto(self.gasto, self.proyecto)
      self.tasa_cambio_id = tc ? tc.id : nil
    end
  end

  # Asigna la subpartida por defecto al gasto_x_proyecto o al gasto segun quien lo haya introducido
  def asigna_subpartida_por_defecto
    # Si es un gasto originado en un proyecto
    if self.gasto.proyecto_origen_id
      # Si el gasto esta creado por este proyecto 
      if self.gasto.proyecto_origen_id == self.proyecto_id
        # Si tenemos una subpartida, probamos a vincularla
        if self.subpartida
          # Si existe la subpartida en el agente, la asignamos 
          subp_agente = Subpartida.find_by_agente_id_and_partida_id_and_nombre(self.gasto.agente_id, self.gasto.partida_id, subpartida.nombre)
          # Actualizamos sin invocar validaciones ni callbacks. No hay peligro con las validaciones de fechas en agente porque en este caso acaba de crearse el gasto
          self.gasto.update_column(:subpartida_agente_id, subp_agente.id) if subp_agente
        end  
      # Si esta creado por otro proyecto
      else
        gxp = GastoXProyecto.find_by_gasto_id_and_proyecto_id(self.gasto_id, self.gasto.proyecto_origen_id)
        # Si existe la subpartida en el proyecto origen, asignamos esa
        if gxp && gxp.subpartida
          # Si existe la subpartida en nuestro proyecto, la asignamos
          subp_nuestra = Subpartida.find_by_proyecto_id_and_partida_id_and_nombre(self.proyecto_id, self.gasto.partida_id, gxp.subpartida.nombre)
          # Actualizamos sin invocar validaciones ni callbacks
          self.update_column(:subpartida_id, subp_nuestra.id) if subp_nuestra
        end
      end
    # Si es un gasto originado en un agente
    else
      # Si existe la subpartida en nuestro proyecto, la asignamos
      if self.gasto.subpartida_agente
        subp_nuestra = Subpartida.find_by_proyecto_id_and_partida_id_and_nombre(self.proyecto_id, self.gasto.partida_id, self.gasto.subpartida_agente.nombre)
        # Actualizamos sin invocar validaciones ni callbacks
        self.update_column(:subpartida_id, subp_nuestra.id) if subp_nuestra
      end
    end
  end

  # Proyecto de la relación.
  def proyecto
    return Proyecto.find_by_id self.proyecto_id
  end

  # Tasa de Cambio asociada
  def tasa_cambio
    tc = TasaCambio.find_by_id(self.tasa_cambio_id)
    return (tc ? tc.tasa_cambio : 0.0)
  end
  # Tasa de Cambio a la divisa 
  def tasa_cambio_divisa
    tc = TasaCambio.find_by_id(self.tasa_cambio_id)
    return (tc ? tc.tasa_cambio_divisa : 0.0)
  end

  # Porcentaje en formato para la visualización
  def porcentaje_to_s
    return (100 * gasto.importe / importe ).to_i.to_s + "%"
  end
  
  #Calcula el importe por la tasa de cambio
  def importe_por_tasa_cambio
    importe_x_tc = importe * tasa_cambio_proyecto.tasa_cambio if tasa_cambio_proyecto && tasa_cambio_proyecto.tasa_cambio
    return ('%.2f' % importe_x_tc) if importe_x_tc
  end

 private

  # Agrupa todas las validaciones para el borrado anteponiendo una variable de clase que permita saber que estamos antes de un borrado
  def verifica_borrado
    @eliminando_gasto = true
    # Agrupamos todos con AND para que el solo se devuelva true si todos devuelven true
    return ( comprobar_estado_proyecto && comprobar_periodos_proyecto && verifica_plugins )
  end

  # Verifica que los plugins permitan la edicion
  # OJO!: Siempre que se hagan movimientos en los gastos desde migraciones hay que tener en cuenta esto!
  def verifica_plugins
    Plugin.activos.each do |plugin|
      begin
        eval(plugin.clase + "::GastoXProyecto").verifica self, @eliminando_gasto
      rescue => ex
      end
    end unless self.evitar_validacion_plugins || (self.changed.empty? && @eliminando_gasto.nil?)
    return self.errors.empty?
  end

end
